/*****************************************************************************
    Ethernet packet sniffer - GreedyDog2

    Developed by Shadow Penguin Security Resarch Group
     http://www.shadowpenguin.org
     UNYUN <unyun@shadowpenguin.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    Thanks : 4lj <4lj@flow.le.to> implements "tmpfile hold" mode
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "depend.h"
#include "gddconfig.h"
#include "param.h"
#include "error.h"
#include "gen.h"
#include "action.h"

/*============================================================================
  Action handler
 [INPUT]
  gddc                  : Configuration of gdd
  filename      : Target logfile name
  sourceIP              : Source IP address
  destIP                : Dest IP address
  sourcePort            : Source port number
  destPort              : Dest port number
  ============================================================================
*/
void do_action(GDDCONFIG *gddc,char *filename,
               char *sourceIP,int sourcePort,char *destIP,int destPort)
{
    char    actiontgt[MAX_PATH],*p;
    char    ssp[64],dsp[64];
    pid_t   pid;

    strncpy(actiontgt,gddc->action,sizeof(actiontgt));
    actiontgt[sizeof(actiontgt)-1]='\0';
    if (!(p=strchr(actiontgt,':'))) return;
    *p=0;
    sprintf(ssp,"%d",sourcePort);
    sprintf(dsp,"%d",destPort);
    if ((pid=fork())==-1){
        errorlog(ERRLOG_FATAL,"fork() error in do_action\n");
        return;
    }else if (pid==0){
        execl(actiontgt,actiontgt,filename,sourceIP,ssp,
              destIP,dsp,gddc->action,NULL);
        errorlog(ERRLOG_FATAL,"Can not exec action '%s'\n",actiontgt);
        exit(0);
    }else
        wait(NULL);
}

/*============================================================================
  Action handler
 [INPUT]
  gddc                  : Configuration of gdd
  filename      : Target logfile name
  sourceIP              : Source IP address
  destIP                : Dest IP address
  sourcePort            : Source port number
  destPort              : Dest port number
 [RETURN]
  0         : Wait async reply
 -1         : ERROR
  ============================================================================
*/
int action(GDDCONFIG *gddc,char *filename,
           char *sourceIP,int sourcePort,char *destIP,int destPort)
{
    pid_t   pid,pid_egrep;
    char    *grepstr;
    int     status;
    char    local_sourceIP[MAXLEN_IPADDR];
    char    local_destIP[MAXLEN_IPADDR];    
    char    local_filename[MAX_PATH];

    /* store to local buffer */
    strncpy(local_sourceIP,sourceIP,MAXLEN_IPADDR);
    strncpy(local_destIP,destIP,MAXLEN_IPADDR);
    strncpy(local_filename,filename,MAX_PATH);
    local_sourceIP[MAXLEN_IPADDR-1]='\0';
    local_destIP[MAXLEN_IPADDR-1]='\0';
    local_filename[MAX_PATH-1]='\0';

    /* If action is not specified,  return now. */
    if (!strlen(gddc->action)) return(-1);
    if ((grepstr=strchr(gddc->action,':'))==NULL) return(-1);
    grepstr++;
    if (*grepstr=='\0') return(-1);

    /* create grep process */
    if ((pid=fork())==-1){
        errorlog(ERRLOG_FATAL,"fork() error\n");
        return(-1);
    }else if (pid==0){

        /* execute egrep and wait */ 
        if ((pid_egrep=fork())==-1){
            errorlog(ERRLOG_FATAL,"fork() error for egrep\n");
            remove(local_filename);
            exit(1);
        }else if (pid_egrep==0){
            execl(EGREP,EGREP,grepstr,local_filename,NULL);
            errorlog(ERRLOG_FATAL,"Can not execute '%s'\n",EGREP);
            exit(1);
        }else{
            status=1;
            wait(&status);
            if (status==0)
                do_action(gddc,local_filename,
                          local_sourceIP,sourcePort,
                          local_destIP,destPort);
            if (gddc->hold==0)
                remove(local_filename);
        }   
        exit(0);
    }
    return(0);
}
