/*****************************************************************************
    Ethernet packet sniffer - GreedyDog2

    Developed by Shadow Penguin Security Resarch Group
     http://www.shadowpenguin.org
     UNYUN <unyun@shadowpenguin.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "depend.h"
#include "gddconfig.h"
#include "param.h"
#include "error.h"
#include "gen.h"
#include "action.h"

/* Parameter to action thread */
struct action_param{
    GDDCONFIG   *gddc;
    char        *filename;
    char        *sourceIP;
    int         sourcePort;
    char        *destIP;
    int         destPort;
};

int thread_lock=0;

/*============================================================================
  Grep & Action
 [INPUT]
  lpData                : Connection data
  ============================================================================
*/
void do_grep(void *lpData)
{
    struct action_param *actparam=(struct action_param *)lpData;
    char                *grepstr;
    char                local_filename[MAX_PATH];
    char                local_sourceIP[MAXLEN_IPADDR];
    char                local_destIP[MAXLEN_IPADDR];
    char                local_action[MAXLEN_ACTION];
    int                 local_sourcePort;
    int                 local_destPort;
    int                 result;
    char                actiontgt[MAX_PATH],*p;
    char                greppfc[MAXLEN_ACTION];
    int                 pfcn,c,er;
    char                ssp[64],dsp[64];

    /* store to local buffer */
    strncpy(local_filename,actparam->filename    ,MAX_PATH);
    strncpy(local_sourceIP,actparam->sourceIP    ,MAXLEN_IPADDR);
    strncpy(local_destIP  ,actparam->destIP      ,MAXLEN_IPADDR);
    strncpy(local_action  ,actparam->gddc->action,MAXLEN_ACTION);
    local_filename[MAX_PATH-1]      ='\0';
    local_sourceIP[MAXLEN_IPADDR-1] ='\0';
    local_destIP[MAXLEN_IPADDR-1]   ='\0';
    local_action[MAXLEN_ACTION-1]   ='\0';
    local_sourcePort = actparam->sourcePort;
    local_destPort   = actparam->destPort;
    thread_lock=0;

    /* Extract grep string */
    if ((grepstr=strchr(local_action,':'))==NULL){
        errorlog(ERRLOG_FATAL,"Invalid action '%s'.\n",
                 local_action);
        remove(local_filename);
        return;
    }
    grepstr++;
    if (*grepstr=='\0'){
        errorlog(ERRLOG_FATAL,"Invalid action '%s'.\n",
                 local_action);
        remove(local_filename);
        return;
    }
    pfcn=1;
    c=er=0;
    strcpy(greppfc,"\"");
    for (pfcn=1;pfcn<sizeof(greppfc)-2;){
        if (grepstr[c]=='\"'){
            greppfc[pfcn]='\\';
            pfcn++;
            if (pfcn==sizeof(greppfc)-2){
                er=1;
                break;
            }
            greppfc[pfcn]='\"';
            pfcn++;
            c++;
        }else{
            greppfc[pfcn]=grepstr[c];
            pfcn++;
            c++;
        }
    }
    if (er){
        errorlog(ERRLOG_FATAL,"Too long action '%s'.\n",
                 local_action);
        remove(local_filename);
        return;
    }
    greppfc[pfcn]='\0';
    strcat(greppfc,"\"");

    /* exec grep.exe */
    result=_spawnl(_P_WAIT,EGREP,EGREP,"-E",greppfc,local_filename,NULL);
    if (result==0){
        strncpy(actiontgt,local_action,sizeof(actiontgt));
        actiontgt[sizeof(actiontgt)-1]='\0';
        if (!(p=strchr(actiontgt,':'))){
            errorlog(ERRLOG_FATAL,"Invalid action '%s'.\n",
                 local_action);
            remove(local_filename);
            return;
        }
        *p=0;
        sprintf(ssp,"%d",local_sourcePort);
        sprintf(dsp,"%d",local_destPort);

        result=_spawnl(_P_WAIT,actiontgt,actiontgt,local_filename,
                       local_sourceIP,ssp,
                       local_destIP,dsp,local_action,NULL);
        if (result==-1)
            errorlog(ERRLOG_FATAL,"Can not exec action '%s'.\n",
                     actiontgt);
    }else if (result==-1)
        errorlog(ERRLOG_FATAL,"Can not exec grep '%s'.\n",EGREP);

    if (actparam->gddc->hold==0)
        remove(local_filename);
}

/*============================================================================
  Action handler
 [INPUT]
  gddc                  : Configuration of gdd
  filename      : Target logfile name
  sourceIP              : Source IP address
  destIP                : Dest IP address
  sourcePort            : Source port number
  destPort              : Dest port number
 [RETURN]
  0         : Wait async reply
 -1         : ERROR
  ============================================================================
*/
int action(GDDCONFIG *gddc,char *filename,
           char *sourceIP,int sourcePort,char *destIP,int destPort)
{
    static struct action_param actparam;
    static char                local_sourceIP[MAXLEN_IPADDR];
    static char                local_destIP[MAXLEN_IPADDR]; 
    static char                local_filename[MAX_PATH];
    int                        i;

    /* If action is not specified, return now. */
    if (!strlen(gddc->action)
     || !strchr(gddc->action,':'))
        return(-1);

    /* Check thread locking */
    if (thread_lock){
        for (i=0;i<100;i++){
            Sleep(100);
            if (!thread_lock) break;
        }
        if (i==100){
            errorlog(ERRLOG_FATAL,"Thread is locked\n");
            thread_lock=0;
            return(-1);
        }
    }

    /* store to local buffer */
    strncpy(local_sourceIP,sourceIP,MAXLEN_IPADDR);
    strncpy(local_destIP  ,destIP  ,MAXLEN_IPADDR);
    strncpy(local_filename,filename,MAX_PATH);
    local_sourceIP[MAXLEN_IPADDR-1] = '\0';
    local_destIP[MAXLEN_IPADDR-1]   = '\0';
    local_filename[MAX_PATH-1]      = '\0';

    actparam.gddc       = gddc;
    actparam.filename   = local_filename;
    actparam.sourceIP   = local_sourceIP;
    actparam.sourcePort = sourcePort;
    actparam.destIP     = local_destIP;
    actparam.destPort   = destPort;

    thread_lock=1;
    if (_beginthread(do_grep,0,&actparam)==-1){
        errorlog(ERRLOG_FATAL,"Can not create thread.\n");
        thread_lock=0;
        return(-1);
    }

    return(0);
}