/*****************************************************************************
    Ethernet packet sniffer - GreedyDog2.1

    Developed by Shadow Penguin Security Resarch Group
     http://www.shadowpenguin.org
     UNYUN <unyun@shadowpenguin.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    Thanks : 4lj <4lj@flow.le.to> implements "tmpfile hold" mode
             xcorp <xcorp@104.net> fixes bug in logging_text()
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <time.h>
#include "depend.h"
#include "param.h"
#include "analyze.h"
#include "gen.h"
#include "error.h"
#include "streamio.h"
#include "action.h"

#ifdef WIN32
#include "gui_win32_msghdlr_main.h"
#endif

/*============================================================================
  Convert date&time string
 [INPUT]
  datetimestr   : Date&Time string
 [RETURN]
  converted date&time string
  ============================================================================
*/
char *timeconv(char *datetimestr)
{
    static char     buf[512];
    unsigned int    i;

    strncpy(buf,datetimestr,sizeof(buf)-1);
    buf[sizeof(buf)-1]='\0';
    for (i=0;i<strlen(datetimestr);i++)
        if (buf[i]==' ') buf[i]='_';
        else if (buf[i]==':') buf[i]='-';
        else if (buf[i]=='\\') buf[i]='x';
        else if (buf[i]=='/') buf[i]='x';
    return(buf);
}

/*============================================================================
 Write log by printable character text format
 [INPUT]
  data          : Sniffed packet
  len           : Length of sniffed packet
  fp            : File pointer
  flag          : 0=text mode, 1=non-filter mode
 [RETURN]
  none
  ============================================================================
*/
void logging_text(char *data,unsigned long len,FILE *fp,int flag)
{
    unsigned long       i;

    if (len==0) return;
    for (i=0;i<len;i++){
        if (flag)
            putc(data[i],fp);
        else{
            if (isprint((int)(data[i]&0xFF)) || data[i]=='\n')
                putc(data[i],fp);
        }
    }
}
/*============================================================================
 Write log by XML like tag text format
 [INPUT]
  data          : Sniffed packet
  len           : Length of sniffed packet
  direction     : 0=Input, 1=Output
  date          : date
  fp            : File pointer
 [RETURN]
  none
  ============================================================================
*/
void logging_tagtext(char *data,unsigned long len,int direction,
                     char *date,FILE *fp)
{
    unsigned long       i;

    if (len==0) return;
    if (direction==0) fprintf(fp,"    <input date=\"%s\">\n",date);
    else              fprintf(fp,"    <output date=\"%s\">\n",date);
    for (i=0;i<len;i++){
        if (data[i]=='<') fprintf(fp,"&lt;");
        else if (data[i]=='>') fprintf(fp,"&gt;");
        else if (data[i]=='\'') fprintf(fp,"&apos;");
        else if (data[i]=='\"') fprintf(fp,"&quot;");
        else if (data[i]=='&') fprintf(fp,"&amp;");
        else if (isprint((int)(data[i]&0xFF)) || data[i]=='\n')
            putc(data[i],fp);
        else fprintf(fp,"&#x00%02X;",data[i]&0xff);
    }
    if (direction==0) fprintf(fp,"    </input>\n");
    else              fprintf(fp,"    </output>\n");
}
/*============================================================================
 Write log by HTML format
 [INPUT]
  data          : Sniffed packet
  len           : Length of sniffed packet
  direction     : 0=Input, 1=Output
  fp            : File pointer
 [RETURN]
  none
  ============================================================================
*/
void logging_HTML(char *data,unsigned long len,int direction,FILE *fp)
{
    unsigned long       i;

    if (len==0) return;
    if (direction==0) fprintf(fp,"<B>");
    for (i=0;i<len;i++){
        if (data[i]=='<') fprintf(fp,"&lt;");
        else if (data[i]=='>') fprintf(fp,"&gt;");
        else if (data[i]=='\"') fprintf(fp,"&quot;");
        else if (data[i]=='&') fprintf(fp,"&amp;");
        else if (isprint((int)(data[i]&0xFF)) || data[i]=='\n')
            putc(data[i],fp);
    }
    if (direction==0) fprintf(fp,"</B>");
}
/*============================================================================
 Write log by hex binary format
 [INPUT]
  data          : Sniffed packet
  len           : Length of sniffed packet
  direction     : 0=Input, 1=Output
  date          : date
  fp            : File pointer
 [RETURN]
  none
  ============================================================================
*/
void logging_binary(char *data,unsigned long len,int direction,
                    char *date,FILE *fp)
{
    char            *dirstr[]={"In","Out"};
    unsigned long   i;

    if (len==0) return;
    fprintf(fp,"direction(%s) : date(%s)\n",dirstr[direction],date);
    for (i=0;i<len;i++){
        fprintf(fp,"%02X ",data[i]&0xff);
        if (((i+1)%16)==0) fprintf(fp,"\n");
    }
    fprintf(fp,"\n");
}

/*============================================================================
 Make filename by IPaddress and portnumber, open the temp logfile
 [INPUT]
  gddc              : Configuration of gdd
  start_time        : Session start time
  logytpe           : Log type
  sourceIP          : Source IP address
  destIP            : Dest IP address
  sourcePort        : Source port number
  destPort          : Dest port number
 [RETURN]
  STREAM_NOERR      : No error
  STREAM_PRMERR     : Parameter error
  STREAM_OPENERR    : open(W) error
  ============================================================================
*/
int streamlogging_open(GDDCONFIG *gddc,
        char *start_time,unsigned char logtype,
        char *sourceIP,unsigned short sourcePort,
        char *destIP,unsigned short destPort)
{
    static char     buf[512];
    FILE            *fp;
    int             i;
    time_t          timeval;
    
    /* store date&time to timeval */
    timeval = time(NULL);

    /* Make filename */
    snprintf(buf,sizeof(buf),"%s%s%s_%s(%u)-%s(%u)",gddc->templogdir,DIRSEPCHR,
        timeconv(start_time),sourceIP,sourcePort,destIP,destPort);
    buf[sizeof(buf)-1]='\0';

    /* Open logfile */
    if ((fp=fopen(buf,"w"))==NULL){
        errorlog(ERRLOG_WARNING,
            "Can not append to temp logfile '%s'\n",buf);
        return(STREAM_OPENERR);
    }
    setperm_logfile(0,buf);

    /* Write header */
    switch(logtype){

    case LOGTYPE_TAGTEXT:
        fprintf(fp,"<session>\n");
        fprintf(fp,"  <ID>%s.%lu</ID>\n",buf,(unsigned long)timeval);
        fprintf(fp,"  <date>%s</date>\n",start_time);
        fprintf(fp,"  <connection>\n");
        fprintf(fp,"    <sourceIP>%s</sourceIP>\n",sourceIP);
        fprintf(fp,"    <sourcePort>%u</sourcePort>\n",sourcePort);
        fprintf(fp,"    <destIP>%s</destIP>\n",destIP);
        fprintf(fp,"    <destPort>%u</destPort>\n",destPort);
        fprintf(fp,"  </connection>\n");
        fprintf(fp,"  <log>\n");
        break;
    case LOGTYPE_HTML:
        fprintf(fp,"<!DOCTYPE HTML PUBLIC \"-//IBM//DTD HPB HTML//EN\">\n");
        fprintf(fp,"<HTML>\n");
        fprintf(fp,"<HEAD>\n");
        fprintf(fp,"  <TITLE>%s</TITLE>\n",buf);
        fprintf(fp,"</HEAD>\n");
        fprintf(fp,"<BODY>\n");
        fprintf(fp,"<PRE>\n");
        fprintf(fp,"ID     : %s.%lu\n",buf,(unsigned long)timeval);
        fprintf(fp,"date   : %s\n",start_time);
        fprintf(fp,"source : %s(%u)\n",sourceIP,sourcePort);
        fprintf(fp,"dest   : %s(%u)\n",destIP,destPort);
        fprintf(fp,"----------------------------------------------\n");
        break;
    case LOGTYPE_BIN:
    case LOGTYPE_TEXT:  
    case LOGTYPE_NONFILTER:  
        fprintf(fp,"\n\n\n");
        for (i=0;i<60;i++) putc('=',fp); fprintf(fp,"\n");
        fprintf(fp,"CONNECTION %s(%u) -> %s(%u)\n",
            sourceIP,sourcePort,destIP,destPort);
        for (i=0;i<60;i++) putc('=',fp); fprintf(fp,"\n");
        break;
    default:
        fclose(fp);
        return(STREAM_PRMERR);
    }
    fclose(fp);
    return(STREAM_NOERR);
}

/*============================================================================
 Copy templogfile to main logfile, remove temp logfile.
 [INPUT]
  gddc              : Configuration of gdd
  start_time        : Session start time
  logytpe           : Log type
  sourceIP          : Source IP address
  destIP            : Dest IP address
  sourcePort        : Source port number
  destPort          : Dest port number
 [RETURN]
  STREAM_NOERR      : No error
  STREAM_OPENERR    : open(W) error
  ============================================================================
*/
int streamlogging_close(GDDCONFIG *gddc, 
        char *start_time,unsigned char logtype,
        char *sourceIP,unsigned short sourcePort,
        char *destIP,unsigned short destPort)
{
    static char buf[512],fbuf[8192];
    FILE        *fp,*fp2;
    int         r;
    
    /* Make filename */
    snprintf(buf,sizeof(buf),"%s%s%s_%s(%u)-%s(%u)", gddc->templogdir,DIRSEPCHR,
        timeconv(start_time),sourceIP,sourcePort,destIP,destPort);
    buf[sizeof(buf)-1]='\0';

    /* Open logfile */
    if ((fp=fopen(buf,"a"))==NULL){
        errorlog(ERRLOG_WARNING,
            "Can not append to temp logfile '%s'\n",buf);
        return(STREAM_OPENERR);
    }
    setperm_logfile(0,buf);

    /* Write footer */
    switch(logtype){

    case LOGTYPE_BIN:
        break;
    case LOGTYPE_TAGTEXT:
        fprintf(fp,"  </log>\n");
        fprintf(fp,"</session>\n");
        break;
    case LOGTYPE_HTML:
        fprintf(fp,"</PRE>\n");
        fprintf(fp,"</BODY>\n");
        fprintf(fp,"</HTML>\n");
        break;
    case LOGTYPE_TEXT:  
        break;
    case LOGTYPE_NONFILTER:  
        break;
    default:
        fclose(fp);
        return(STREAM_PRMERR);
    }
    fclose(fp);

    /* Copy templog to main logfile */
    if ((fp=fopen(gddc->logfile,"a"))!=NULL){
        setperm_logfile(0,gddc->logfile);
        if ((fp2=fopen(buf,"r"))!=NULL){
            for (;;){
                r=fread(fbuf,1,sizeof(fbuf),fp2);
                if (r<=0) break;
                fwrite(fbuf,1,r,fp);
            }
            fclose(fp2);
        }else
            errorlog(ERRLOG_WARNING,
                "Can not read temp logfile '%s'\n",buf);
        fclose(fp);
    }else
        errorlog(ERRLOG_WARNING,
              "Can not append to logfile '%s'\n",gddc->logfile);

    /* Check action & remove */
    if (action(gddc,buf,sourceIP,sourcePort,destIP,destPort))
        if (gddc->hold==0) remove(buf);

#ifdef WIN32
    Win32MsgSubfunc_AddList(gddc,start_time,sourceIP,sourcePort,destIP,destPort,logtype);
#endif

    return(STREAM_NOERR);
}

/*============================================================================
 Write log to temp logfile
 [INPUT]
  gddc              : Configuration of gdd
  start_time        : Session start time
  data              : Data to write
  length            : Length of data
  direction         : Packet direction
  logytpe           : Log type
  sourceIP          : Source IP address
  destIP            : Dest IP address
  sourcePort        : Source port number
  destPort          : Dest port number
 [RETURN]
  STREAM_NOERR      : No error
  STREAM_OPENERR    : open(W) error
  ============================================================================
*/
int streamlogging_write(GDDCONFIG *gddc,char *start_time,
        char *data,unsigned long length,
        int direction,unsigned char logtype,
        char *sourceIP,unsigned short sourcePort,
        char *destIP,unsigned short destPort)
{
    static char buf[512];
    static char datestr[512];
    FILE        *fp;
    time_t      timeval;
    struct tm   *timep=NULL;
    char        *timesp=NULL;
    char        *c;

    /* store date&time to datestr */
    timeval=time(NULL);
    if (timeval!=(time_t)-1){
        timep=localtime(&timeval);
        if (timep!=NULL)
        timesp=asctime(timep);
    }
    if (timesp==NULL)
        strcpy(datestr,"time() error ");
    else{
        strncpy(datestr,timesp,sizeof(datestr)-1);
        datestr[sizeof(datestr)-1]='\0';
        if ((c=strchr(datestr,'\r'))!=NULL
         || (c=strchr(datestr,'\n'))!=NULL) *c='\0';
        }

    /* Make filename */
    snprintf(buf,sizeof(buf),"%s%s%s_%s(%u)-%s(%u)",gddc->templogdir,DIRSEPCHR,
        timeconv(start_time),sourceIP,sourcePort,destIP,destPort);
    buf[sizeof(buf)-1]='\0';

    /* Open logfile */
    if ((fp=fopen(buf,"a"))==NULL){
        errorlog(ERRLOG_WARNING,
            "Can not append to temp logfile '%s'\n",buf);
        return(0);
    }
    setperm_logfile(0,buf);

    /* Write data */
    switch(logtype){

    case LOGTYPE_BIN:
        logging_binary(data,length,direction,datestr,fp);
        break;
    case LOGTYPE_TAGTEXT:
        logging_tagtext(data,length,direction,datestr,fp);
        break;
    case LOGTYPE_HTML:
        logging_HTML(data,length,direction,fp);
        break;
    case LOGTYPE_TEXT:  
        logging_text(data,length,fp,0);
        break;
    case LOGTYPE_NONFILTER:  
        logging_text(data,length,fp,1);
        break;
    default:
        fclose(fp);
        return(STREAM_PRMERR);
    }
    fclose(fp);
    return(STREAM_NOERR);
}
/*============================================================================
 Write log to logfile directory
 [INPUT]
  gddc              : Configuration of gdd
  msg               : Message String
  data              : Data to write
  length            : Length of data
  direction         : Packet direction
  sourceIP          : Source IP address
  destIP            : Dest IP address
  sourcePort        : Source port number
  destPort          : Dest port number
 [RETURN]
  STREAM_NOERR      : No error
  STREAM_OPENERR    : open(W) error
  ============================================================================
*/
int streamlogging_interrupt(GDDCONFIG *gddc,char *msg,char *data,
        unsigned long length,int direction,
        char *sourceIP,unsigned short sourcePort,
        char *destIP,unsigned short destPort)
{
    static char     datestr[512];
    time_t          timeval;
    struct tm       *timep=NULL;
    char            *timesp=NULL;
    char        *c;
    FILE            *fp;
    unsigned long   i,j;

    /* store date&time to datestr */
    timeval=time(NULL);
    if (timeval!=(time_t)-1){
        timep=localtime(&timeval);
        if (timep!=NULL)
            timesp=asctime(timep);
    }
    if (timesp==NULL)
        strcpy(datestr,"time() error ");
    else{
        strncpy(datestr,timesp,sizeof(datestr)-1);
        datestr[sizeof(datestr)-1]='\0';
        if ((c=strchr(datestr,'\r'))!=NULL
         || (c=strchr(datestr,'\n'))!=NULL) *c='\0';
    }

    /* Open and logging */
    if ((fp=fopen(gddc->errlog,"a"))==NULL){
        errorlog(ERRLOG_WARNING,
              "Can not append to logfile '%s'\n",gddc->logfile);
        return(STREAM_OPENERR);
    }
    fprintf(fp,"\n");
    fprintf(fp,"%s %s(%u)->%s(%u) Length=%lu Date=%s\n",msg,
        sourceIP,sourcePort,destIP,destPort,length,datestr);
    for (i=0;i<length;i+=16){
        for (j=0;j<16;j++){
            if (i+j>=length) break;
            fprintf(fp,"%02X ",data[i+j]&0xff);
        }
        fprintf(fp,"\n");
    }
    fprintf(fp,"\n");
    fclose(fp);
    return(STREAM_NOERR);
}
